<?php
namespace Teknisa\PLA\Modules\Cardapio\Services;

use Zeedhi\Framework\DataSource\FilterCriteria;
use \Setforms2\Shared\ORM\DateTime;
use \Setforms2\Shared\ORM\TimeZone;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class Pla07700AprovacaoCardapioMatrizService {

    protected $dataSourceManager;
    protected $environment;
    protected $entityManager;
    protected $connection;
    protected $newCode;
    protected $authService;

	public function __construct($dataSourceManager, $environment, $entityManager, $newCode, $authService) {
        $this->dataSourceManager = $dataSourceManager;
        $this->environment       = $environment;
        $this->entityManager     = $entityManager;
        $this->connection        = $this->entityManager->getConnection();
        $this->newCode           = $newCode;
        $this->authService       = $authService;
    }


    /**
    * Função resposável por retornar o campo observação geral para o 
    * requisitante do serviço.
    * 
    * @param {String} $dtMesCard Data filtrada para o cardápio.
    * 
    * @return {Object} Resultado da consulta de observação geral de acordo com
    *   os dados filtrados.
    */
    public function getDsgeraprocardmatr($dtMesCard){
        
        $sql = "SELECT DSGERAPROCARDMATR
                    FROM   OBSAPROVCARDMATR
                    WHERE  DTMESCARDAPROVMATR = :P_DTMESCARD";
        
        $params = array(
            "P_DTMESCARD" => $dtMesCard
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
        
    }
    
    
    /**
    * Função responsável por retornar um novo número de lista para ser usado.
    * 
    * @return {String} Número da lista gerada.
    */
    public function getNrLista(){
        return $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrOrg());
    }
    
    
    /**
    * Função responsável por inserir um novo registro na tabela lista.
    * 
    * @param {String} $cdLista Dado a ser gravado na lista.
    * @param {String} $nrlista Número da lista a ser usada.
    */
    public function insertLista($cdlista, $nrlista) {

        $lista = new \Geral\Entities\Lista();
        $lista = $this->buildLista($lista, $nrlista, $cdlista);
        
        $this->entityManager->persist($lista);
        $this->entityManager->flush();
        
    }


    /**
    * Função responsável por remover um registro da tabela lista.
    * 
    * @param {String} $nrlista Dado a ser removido da lista.
    */
    public function removeLista($nrlista) {
        
        $params = array(
            'nrlista' => $nrlista,
            'nrorg' => $this->environment->getNrOrg(),
        );
        
        $listas = $this->entityManager->getRepository('Geral\Entities\Lista')->findBy($params);
        
        foreach ($listas as $lista) {
            
            $this->entityManager->remove($lista);
            $this->entityManager->flush();
        }

    }
    
    
    /**
    * Função responsável por construir um novo objeto da tabela lista.
    * 
    * @param {Object} $lista Objeto list a ser montado.
    * @param {String} $nrlista Dado a ser gravado na lista.
    * @param {String} $cdLista Código da Lista a ser utilizado.
    * 
    * @return {Object} Lista construida a ser retornada.
    */
    protected function buildLista($lista, $nrlista, $cdlista) {
        
        $lista->setNrlista($nrlista);
        $lista->setCdlista($cdlista);
        $lista->setDtultatulist(new DateTime());
        
        $lista->setNrorg($this->environment->getNrOrg());
        $lista->setDtinclusao(new DateTime());
        $lista->setDtultatu(new DateTime());
        $lista->setCdoperinclusao($this->environment->getCdOperador());
        $lista->setIdativo('S');
        
        return $lista;
    }
    
    
    /**
    * Função responsável por retornar cardapio de acordo com o filtro passado
    * na lista e da data do cardápio.
    * 
    * @param {String} $nrlista Número da lista onde estão gravados os dados
    *   usados para filtrar.
    * @param {String} $dtMesCard Data do cardápio usada para filtrar dados.
    * @param {Array} $paramsToExecute Paramêtros para execução da query..
    * 
    * @return {Object} Resultado da consulta de cardápios de acordo com os 
    *   dados filtrados.
    */
    public function getCardapio($nrlista, $dtMesCard, $paramsToExecute){
        $sql = "
        SELECT 'N' AS IDMARCADO, S.CDFILIAL, F.NMFILIAL, S.CDSERVFILI,
            S.NMSERVICO, CTRL.DTAPROCARDFILI, CTRL.CDOPERAPROCARDMATR,
            O.NMOPERADOR, CTRL.DTAPROCARDMATR, CTRL.DSOBSAPROCARDMATR,
            CTRL.DTMESCARDAPROV, C.DTLIMAPROVMATR,
            NVL(PP.IDRESTRAPROVCARD,'N') AS IDRESTRAPROVCARD
        FROM
            LISTA L                    
            INNER JOIN SERVICO S ON S.CDFILIAL = SUBSTR(L.CDLISTA, 1, 4) AND
                                    S.CDSERVFILI = SUBSTR(L.CDLISTA, 6, 10) 
            INNER JOIN FILIAL F ON F.CDFILIAL = S.CDFILIAL
            LEFT JOIN PARAMPLANEJA PP ON PP.CDFILIAL = F.CDFILIAL
            INNER JOIN CALENCARD C ON C.CDFILIAL = F.CDFILIAL AND C.DTMESCALENCARD = :P_DTMESCARD
            INNER JOIN CTRLAPROVCARD CTRL ON CTRL.CDFILIAL = S.CDFILIAL AND
                                             CTRL.CDSERVFILI = S.CDSERVFILI AND
                                             CTRL.DTMESCARDAPROV = C.DTMESCALENCARD
            LEFT JOIN OPERADOR O ON O.CDOPERADOR = CDOPERAPROCARDMATR                    
        WHERE L.NRLISTA = :NRLISTA
          AND S.IDSERVATIV = 'S'
          AND S.IDCARDDESCENTR = 'S'
          AND (
                CTRL.DTAPROCARDFILI IS NOT NULL
                OR TRUNC(SYSDATE) > C.DTLIMAPROVFILI + NVL(C.QTDIAPRORAPROV,0)
            )                    
        ORDER BY
            S.CDFILIAL,
            S.CDSERVFILI
        ";
        
        $params = array(
            'NRLISTA' => $nrlista,
            'P_DTMESCARD' => $dtMesCard
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }
    
    
    /**
    * Função responsável por consultar e retornar true caso haja restrições 
    * contratuais de  acordo com os parâmetros passados.
    * 
    * @param {String} $cdFilial Código da unidade a ser verificada.
    * @param {String} $cdServFili Código do serviço a ser verificado.
    * @param {String} $mesCard Mes do cardápio a ser verificado.
    * 
    * @return {Boolean} Retorno de true caso exista restrições e false caso não
    *   exista.
    */
    public function isRestrictedCard($cdFilial, $cdServFili, $mesCard) {
        
        $sql = "SELECT DISTINCT CDFILIAL, CDSERVFILI
            FROM TABLE (RESTR_CONTRATUAIS_CARD(:P_CDFILIAL, :P_CDSERVFILI, NULL, :P_DTINICIAL, :P_DTFINAL))";

        //Cria variáveis de data.
        $newDate = DateTime::createFromFormat('d/m/Y', '01/'.$mesCard)->format('Y-m-d');
        $firstDay = '01/'.$mesCard;
        $lastDay = date('t', strtotime($newDate)).'/'.$mesCard;

        $params = array(
            "P_CDFILIAL" => $cdFilial,
            "P_CDSERVFILI" => $cdServFili,
            "P_DTINICIAL" => $firstDay,
            "P_DTFINAL" => $lastDay
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        
        $result = $query->fetchAll();
        
        return isset($result[0]['CDFILIAL'])&&isset($result[0]['CDSERVFILI']);
        
    }
    
    
    /**
    * Função responsável por consultar e retornar true caso o cardápio passado
    * por parâmetro já esteja aprovado pela filial.
    * 
    * @param {String} $cdFilial Código da unidade a ser verificada.
    * @param {String} $cdServFili Código do serviço a ser verificado.
    * @param {String} $mesCard Mes do cardápio a ser verificado.
    * 
    * @return {Boolean} Retorno de true caso cardápio esteja aprovado e false
    *   caso não esteja.
    */
    public function isAprovedCardFilial($cdFilial, $cdServFili, $mesCard) {
        
        $sql = "SELECT DTAPROCARDFILI
                    FROM   CTRLAPROVCARD
                    WHERE  CDFILIAL = :P_CDFILIAL AND
                           CDSERVFILI = :P_CDSERVICO AND
                           DTMESCARDAPROV = :P_DTMESCARD";
        
        $params = array(
            "P_CDFILIAL" => $cdFilial,
            "P_CDSERVICO" => $cdServFili,
            "P_DTMESCARD" => '01/'.$mesCard
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll()[0];
        
        return isset($result['DTAPROCARDFILI']);
        
    }
    
    
    /**
    * Função responsável por aprovar cardápios genericamente
    * sem restrição contratual.
    * 
    * @param {Array} $register Registro atual para verificar.
    * @param {Array}  $dataToSave Dados usados para aprovar cardápio.
    */
    public function aprovarCard($register, $dataToSave) {
        
        $sqlAproveCard = "UPDATE CTRLAPROVCARD
                    SET    
                        CDOPERAPROCARDMATR  = :CDOPERAPROCARDMATR,
                        DTAPROCARDMATR      = :DTAPROCARDMATR,
                        DSOBSAPROCARDMATR   = :DSOBSAPROCARDMATR
                    WHERE  
                        CDFILIAL       = :CDFILIAL   AND
                        CDSERVFILI     = :CDSERVFILI AND
                        DTMESCARDAPROV = :DTMESCARDAPROV AND 
                        DTAPROCARDMATR IS NULL";

        $sqlSaveObs = "UPDATE CTRLAPROVCARD
                    SET    
                        DSOBSAPROCARDMATR   = :DSOBSAPROCARDMATR
                    WHERE  
                        CDFILIAL       = :CDFILIAL   AND
                        CDSERVFILI     = :CDSERVFILI AND
                        DTMESCARDAPROV = :DTMESCARDAPROV";

        
        $params = array(
            "CDFILIAL" => $register['CDFILIAL'],
            "CDSERVFILI" => $register['CDSERVFILI'],
            "DTMESCARDAPROV" => $register['DTMESCARDAPROV'],
            "DSOBSAPROCARDMATR" => $register['DSOBSAPROCARDMATR'],
            "DTAPROCARDMATR" => date_format(new DateTime(), 'd/m/Y H:i:s'),
            "CDOPERAPROCARDMATR" => $this->environment->getCdOperador()
        );
        
        $query = $this->connection->prepare($sqlAproveCard);
        $query->execute($params);

        unset($params['CDOPERAPROCARDMATR']);
        unset($params['DTAPROCARDMATR']);
        $query = $this->connection->prepare($sqlSaveObs);
        $query->execute($params);
        
        $register['IDSITUOPERCARD'] = 'S';
        $this->recordLog($register);
    }
    

    /**
    * Função responsável por concluir a aprovação de cardápios
    * que já foram aprovados pela filial.
    * 
    * @param {Array} $register Registro atual para aprovar.
    * @param {Array}  $dataToSave Dados usados para aprovar cardápio.
    */
    public function aprovarCardAprovadosFilial($register, $dataToSave){
        $sql = "UPDATE CTRLAPROVCARD
                    SET    
                        CDTIPOCUSTOMATR     = :CDTIPOCUSTOMATR,
                        VRPUCTRLAPRCARDMATR = :VRPUCTRLAPRCARDMATR
                    WHERE  
                        CDFILIAL       = :CDFILIAL   AND
                        CDSERVFILI     = :CDSERVFILI AND
                        DTMESCARDAPROV = :DTMESCARDAPROV AND 
                        DTAPROCARDMATR IS NULL";

        
        $params = array(
            "CDFILIAL" => $register['CDFILIAL'],
            "CDSERVFILI" => $register['CDSERVFILI'],
            "DTMESCARDAPROV" => $register['DTMESCARDAPROV'],
            "VRPUCTRLAPRCARDMATR" => $register['VRCUSTPRODMAT'],
            "CDTIPOCUSTOMATR" => $dataToSave['TIPOCUSTO']
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }

    /**
    * Função responsável por concluir a aprovação de cardápios
    * cuja data limite de aprovação pela filial está expirada.
    * 
    * @param {Array} $register Registro atual para aprovar.
    * @param {Array}  $dataToSave Dados usados para aprovar cardápio.
    */
    public function aprovarCardExpiradosFilial($register, $dataToSave){
        $sql = "UPDATE CTRLAPROVCARD
                    SET    
                        CDOPERAPROCARDFILI = :CDOPERAPROCARDFILI,
                        DTAPROCARDFILI     = :DTAPROCARDFILI
                    WHERE  
                        CDFILIAL       = :CDFILIAL   AND
                        CDSERVFILI     = :CDSERVFILI AND
                        DTMESCARDAPROV = :DTMESCARDAPROV AND 
                        DTAPROCARDMATR IS NULL";

        
        $params = array(
            "CDFILIAL" => $register['CDFILIAL'],
            "CDSERVFILI" => $register['CDSERVFILI'],
            "DTMESCARDAPROV" => $register['DTMESCARDAPROV'],
            "DTAPROCARDFILI" => date_format(new DateTime(), 'd/m/Y H:i:s'),
            "CDOPERAPROCARDFILI" => $this->environment->getCdOperador()
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    
    /**
    * Função responsável por cancelar o cardápio.
    * 
    * @param {Array} $register Registro atual para cancelar.
    * @param {Array}  $dataToSave Dados usados para aprovar cardápio.
    */
    public function cancelarCard($register, $dataToSave){
        $sql = "UPDATE CTRLAPROVCARD
                SET    CDOPERAPROCARDMATR  = :CDOPERAPROCARDMATR,
                    DTAPROCARDMATR      = :DTAPROCARDMATR,
                    DSOBSAPROCARDMATR   = :DSOBSAPROCARDMATR,
                    CDTIPOCUSTOMATR     = :CDTIPOCUSTOMATR,
                    VRPUCTRLAPRCARDMATR = :VRPUCTRLAPRCARDMATR
                WHERE  CDFILIAL   = :CDFILIAL   AND
                    CDSERVFILI = :CDSERVFILI AND
                    DTMESCARDAPROV = :DTMESCARDAPROV AND
                    DTAPROCARDMATR IS NOT NULL";

        
        $params = array(
            "CDFILIAL" => $register['CDFILIAL'],
            "CDSERVFILI" => $register['CDSERVFILI'],
            "DTMESCARDAPROV" => $register['DTMESCARDAPROV'],
            "DSOBSAPROCARDMATR" => $register['DSOBSAPROCARDMATR'],
            "DTAPROCARDMATR" => "",
            "CDOPERAPROCARDMATR" => $this->environment->getCdOperador(),
            "VRPUCTRLAPRCARDMATR" => "",
            "CDTIPOCUSTOMATR" => ""
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);

        $register['IDSITUOPERCARD'] = 'N';
        $this->recordLog($register);
    }

    /**
    * Função responsável por checar se a data limite de aprovacao do cardapio
    * pela filial está expirada para o cardápio passado por parâmetro.
    *
    * @param {String} $cdFilial - Código da filial a ser verificada.
    * @param {String} $dtMesCard - Mes do cardapio a ser verificado.
    *
    * @return {Boolean} True se a data limite estiver expirada
    *   para o cardapio passado por parametro.
    */
    public function isExpiredDataLimiteAprovacaoFilial($cdFilial, $dtMesCard){
        $sql = "SELECT DTLIMAPROVFILI + NVL(QTDIAPRORAPROV, 0) AS DATALIMITE
                FROM   CALENCARD
                WHERE  CDFILIAL = :P_CDFILIAL AND
                    DTMESCALENCARD = :P_DTMESCARD";

        $params = array(
            "P_CDFILIAL" => $cdFilial,
            "P_DTMESCARD" => $dtMesCard
        );

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll()[0]["DATALIMITE"];

        $limitDate = DateTime::createFromFormat('d/m/Y H:i:s', $result)->format('Y-m-d');
        $currDate = date('Y-m-d');

        return strtotime($limitDate)<strtotime($currDate);
    }
    
    
    /**
    * Função responsável por checar se a data limite de aprovacao do cardapio
    * pela matriz está expirada para o cardápio passado por parâmetro.
    *
    * @param {String} $cdFilial - Código da filial a ser verificada.
    * @param {String} $dtMesCard - Mes do cardapio a ser verificado.
    *
    * @return {Boolean} True se a data limite estiver expirada
    *   para o cardapio passado por parametro.
    */
    public function isExpiredDataLimiteAprovacaoMatriz($cdFilial, $dtMesCard){
        $sql = "SELECT DTLIMAPROVMATR
                    FROM   CALENCARD
                    WHERE  CDFILIAL = :P_CDFILIAL AND
                        DTMESCALENCARD = :P_DTMESCARD";

        $params = array(
            "P_CDFILIAL" => $cdFilial,
            "P_DTMESCARD" => $dtMesCard
        );

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll()[0]["DTLIMAPROVMATR"];

        $limitDate = DateTime::createFromFormat('d/m/Y H:i:s', $result)->format('Y-m-d');
        $currDate = date('Y-m-d');

        return strtotime($limitDate)<strtotime($currDate);
    }

    /**
    * Função responsável por salvar o campo de observação geral.
    * 
    * @param {String} $obsGer Observação geral alterada.
    * @param {String} $dtMesCard Data mes do cardapio.
    */
    public function saveObservGer($obsGer, $dtMesCard){
        
        $sqlQueries = array(
            "DELETE FROM OBSAPROVCARDMATR
                WHERE DTMESCARDAPROVMATR = :P_DTMESCARDAPROVMATR",    
            "INSERT INTO OBSAPROVCARDMATR
                (DTMESCARDAPROVMATR, DSGERAPROCARDMATR)
            VALUES
                (:P_DTMESCARDAPROVMATR, :P_DSGERAPROCARDMATR)"
        );
                    
        $params = array(
            "P_DTMESCARDAPROVMATR" => $dtMesCard
        );
        
        $deleteQuery = $this->connection->prepare($sqlQueries[0]);
        $deleteQuery->execute($params);
        
        $params["P_DSGERAPROCARDMATR"] = $obsGer;

        $insertQuery = $this->connection->prepare($sqlQueries[1]);
        $insertQuery->execute($params);
        
    }

    /**
    * Função responsável por retornar o NrConsplan do cardapio passado
    * por parâmetro.
    *
    * @param {String} $nrLista Numero da lista a ser utilizada.
    * @param {Array} $dataToExecute Dados para execução da procedure.
    */
    public function getConsPlanNumber($nrLista, $dataToExecute){
        $getConsPlanNum = new StoredProcedure($this->connection, 'CONSPLAN_GERAL');
        $getConsPlanNum->addParam(new Param('P_NRLISTA', Param::PARAM_INPUT, $nrLista, Param::PARAM_TYPE_STR, strlen($nrLista)));
        $getConsPlanNum->addParam(new Param('P_DTINICIAL', Param::PARAM_INPUT, $dataToExecute['DT_INICIAL'], Param::PARAM_TYPE_STR, $dataToExecute['DT_INICIAL']));
        $getConsPlanNum->addParam(new Param('P_DTFINAL', Param::PARAM_INPUT, $dataToExecute['DT_FINAL'], Param::PARAM_TYPE_STR, $dataToExecute['DT_INICIAL']));
        $getConsPlanNum->addParam(new Param('P_IDPRODUTOS', Param::PARAM_INPUT, 'CFP', Param::PARAM_TYPE_STR, 3));
        $getConsPlanNum->addParam(new Param('P_IDSEPPRATO', Param::PARAM_INPUT, 'P', Param::PARAM_TYPE_STR, 1));
        $getConsPlanNum->addParam(new Param('P_PEACRESCIMO', Param::PARAM_INPUT, 0, Param::PARAM_TYPE_INT));
        $getConsPlanNum->addParam(new Param('P_IDARREDOND', Param::PARAM_INPUT, 'N', Param::PARAM_TYPE_STR, 1));
        $getConsPlanNum->addParam(new Param('P_CDPRODUTOINI', Param::PARAM_INPUT, '0', Param::PARAM_TYPE_STR, 1));
        $getConsPlanNum->addParam(new Param('P_CDPRODUTOFIN', Param::PARAM_INPUT, 'zzzzzzzzzz', Param::PARAM_TYPE_STR, 10));
        $getConsPlanNum->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $dataToExecute['CDOPERADOR'], Param::PARAM_TYPE_STR, 12));
        $getConsPlanNum->addParam(new Param('P_NRCONSPLANTH', Param::PARAM_INPUT, 'GERANOVCOD', Param::PARAM_TYPE_STR, 10));
        $getConsPlanNum->addParam(new Param('P_NRCONSPLAN', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 10));

        return $getConsPlanNum->execute();
    }


    /**
    * Função responsável por retornar o VRPUCTRLAPRCARDMATR de acordo
    * com os parâmetros passados.
    * 
    * @param {String} $nrConsPlan Numero ConsPlan retornado pela procedure.
    * @param {Array} $dataToExecute Dados para executar procedure.
    */
    public function getVrpuCtrlAprCardMatr($nrConsPlan, $dataToExecute){
        $sql = "SELECT 
                    CUST.CDFILIAL, CUST.CDSERVFILI, VRCUSTPRODMAT
                FROM CUSTPADR, CTRLAPROVCARD CT,
                    (SELECT CDFILICPLA AS CDFILIAL, CDSERVCPLA AS CDSERVFILI, 
                            DTMESCARD, VRPEMAXDIFPLAPA, VRPEMINDIFPLAPA, 
                            :P_CDTIPOCUSTO AS CDTIPOCUSTODESC,
                            DECODE(SUM(DECODE(PF.IDCOMCUSPERC, 'C', QTCLIESTICAR, QTCOMESTICAR)), 0, 0, SUM(VRCUSTO)/ SUM(DECODE(PF.IDCOMCUSPERC, 'C', QTCLIESTICAR, QTCOMESTICAR))) AS VRCUSTPRODMAT
                    FROM CARDADIA CA, PARAMFILIAL PF,
                    (SELECT CP.CDFILICPLA, CP.CDSERVCPLA, CP.DTCONSPLAN, PP.VRPEMAXDIFPLAPA, 
                        PP.VRPEMINDIFPLAPA, 
                        SUM(CUSTO_PRODUTO(CP.CDFILICPLA, CP.CDPRODCPLA, :P_CDTIPOCUSTO, NVL(CM.DTREFECUSTMES, CP.DTCONSPLAN), SYSDATE) * CP.QTCONSPLAN) AS VRCUSTO,
                        :P_CDTIPOCUSTO AS CDTIPOCUSTODESC
                    FROM CONSPLAN CP, PARAMPLANEJA PP, CARDADIA CA, CARDAMES CM
                    WHERE (CP.NRCONSPLAN = :P_NRCONSPLAN)
                        AND (INSTR(PP.IDCONSPROD, CP.IDCONSPLAN) <> 0)
                        AND (PP.CDFILIAL = CP.CDFILICPLA)
                        AND (CP.CDFILICPLA = CA.CDFILIAL)
                        AND (CP.CDSERVCPLA = CA.CDSERVFILI)
                        AND (CP.DTCONSPLAN = CA.DTCARDAPIO)
                        AND (CA.CDFILIAL = CM.CDFILIAL (+))
                        AND (CA.CDSERVFILI = CM.CDSERVFILI (+))
                        AND (CA.DTMESCARD = CM.DTMESCARDAPIO (+))
                    GROUP BY CP.CDFILICPLA, CP.CDSERVCPLA, CP.DTCONSPLAN, VRPEMAXDIFPLAPA,
                    VRPEMINDIFPLAPA, CDTIPOCUSTODESC) PROD
                    WHERE (PROD.CDFILICPLA = CA.CDFILIAL)
                        AND (PROD.CDSERVCPLA = CA.CDSERVFILI)
                        AND (PROD.DTCONSPLAN = CA.DTCARDAPIO)
                        AND (CA.DTCARDAPIO BETWEEN :P_DTINICIAL and :P_DTFINAL)
                        AND (CA.CDFILIAL = PF.CDFILIAL)
                    GROUP BY CDFILICPLA, CDSERVCPLA, DTMESCARD, VRPEMAXDIFPLAPA,
                    VRPEMINDIFPLAPA, CDTIPOCUSTODESC) CUST
                WHERE (CUST.CDFILIAL = CT.CDFILIAL)
                AND (CUST.CDSERVFILI = CT.CDSERVFILI)
                AND (CUST.DTMESCARD = CT.DTMESCARDAPROV)
                AND (CUST.CDFILIAL = CUSTPADR.CDFILIAL(+))
                AND (CUST.CDSERVFILI = CUSTPADR.CDSERVFILI(+))
                AND (CUST.DTMESCARD = CUSTPADR.DTREFCUSPADR(+))
                AND (NOT (CT.DTLIBECARDMATR IS NULL))";
                    
        $params = array(
            "P_NRCONSPLAN"      => $nrConsPlan,
            "P_DTFINAL"         => $dataToExecute['DT_FINAL'],
            "P_DTINICIAL"       => $dataToExecute['DT_INICIAL'],
            "P_CDTIPOCUSTO"     => $dataToExecute['TIPOCUSTO']
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }

    /**
    * Função responsável por gravar log ao aprovar ou cancelar cardapio.
    *
    * @param {Array} $register - Dados para gravar log.
    */
    private function recordLog($register){
        $sql = "INSERT INTO LOGAPROVCARD
                    (CDFILIAL, CDSERVFILI, DTMESCARDAPROVLOG, 
                    DTLOGAPROVCARD, CDOPERLOGAPROVCARD, 
                    IDSITUOPERCARD, IDTIPOOPERCARD)
                VALUES
                    (:P_CDFILIAL, :P_CDSERVFILI, 
                    :P_DTMESCARDAPROVLOG, :P_DTLOGAPROVCARD, 
                    :P_CDOPERLOGAPROVCARD, :P_IDSITUOPERCARD, 
                    :P_IDTIPOOPERCARD)";

        
        $params = array(
            "P_CDFILIAL" => $register['CDFILIAL'],
            "P_CDSERVFILI" => $register['CDSERVFILI'],
            "P_DTMESCARDAPROVLOG" => $register['DTMESCARDAPROV'],
            "P_DTLOGAPROVCARD" => date_format(new DateTime(), 'd/m/Y H:i:s'),
            "P_CDOPERLOGAPROVCARD" => $this->environment->getCdOperador(),
            "P_IDSITUOPERCARD" => $register['IDSITUOPERCARD'],
            "P_IDTIPOOPERCARD" => 'M'
        );
        
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function getOperadorLibMatr() {
        $row = array(
            'CDOPERADOR'   => $this->environment->getCdOperador(),
            'AUTENTICACAO' => array(
    			'COLUMN_NAME'=> null,
    			'ALIAS_NAME'=> 'LRCD'
		    )
        );
        $permission = $this->authService->returnOperatorInGroup($row);

        return $permission;
    }
    
    public function getOperadorForaMeta() {
        $row = array(
            'CDOPERADOR'   => $this->environment->getCdOperador(),
            'AUTENTICACAO' => array(
    			'COLUMN_NAME'=> null,
    			'ALIAS_NAME'=> 'LCMR'
		    )
        );
        $permission = $this->authService->returnOperatorInGroup($row);

        return $permission;
    }
}
